"use strict";
this.name		= "Ship_Storage_Helper.js";
this.author		= "Capt. Murphy";
this.copyright	= "2012 - Capt. Murphy";
this.license    = "CC BY-NC-SA 3.0"; // see http://creativecommons.org/licenses/by-nc-sa/3.0/ for more info. This OXP is intended to be developed by the OXPer community to ensure maximum OXP compatibility with OXPs that need to store/restore the player ship.
this.credits	= "A big thank-you to cim for coding the new JS commands necessary for changing player ships by script."
this.description= "Worldscript with helper functions to store/restore player ship characteristics as a variable.";
this.version	= "0.22";

this.cargoType = ["food", "textiles", "radioactives", "slaves", "liquorWines", "luxuries", "narcotics", "computers", "machinery", "alloys", "firearms", "furs", "minerals", "gold", "platinum", "gemStones", "alienItems"];
this.loggingEnabled = false;
this.expectedLength = 28; // expected length of storedShipArray with current release. Must be incremented as new items are added. Used to add extra data on restore if necessary for backwards compatibility as this OXP develops.
this.dataKeyCounter = 128; //old variabe, now use dataKeyMaxIt //number of loop iterations used to build Array of available player dataKeys. A higher number of iterations will result in a more comprehensive list of unique dataKeys being identified (particularly if a lot of different player flyable ship OXPs are installed), but increases the chance of JS timeout/long startUp times on slower computers. However if there is timeout it will try again with a reduced number of iterations on the missionScreenOpportunity that fires shortly after startUp. Should be a power of 2.  Default value of 128 results in reasonable startUp delay on old WinXP test machine and captures around 90-95% of dataKeys when there are 60 player flyable ships actually installed. Win 7 machine can easily cope with figure of upto to 4096, with no noticeable delay
this.dataKeyMaxIt = 5; //max. iteration in Oolite v1.77, increase if you have many OXP ships, reduce if cause "Universe is full" or crash to desktop
this.dataKeyArray = new Array; // will be populated with valid player flyable dataKeys from the users currently installed OXPs at this.startUp.

this.startUp = function()
{
	if (this.loggingEnabled){log(this.name,"Building player dataKey Array.");}
	this.buildDataKeyArray(this.dataKeyMaxIt);
}

this.missionScreenOpportunity = function()
{
	if (this.dataKeyArrayBuilt){return;}
	if (this.loggingEnabled){log(this.name,"Retry building player dataKey Array.");}
	this.dataKeyMaxIt = this.dataKeyMaxIt+1;
	this.buildDataKeyArray(this.dataKeyMaxIt);
}

this.buildDataKeyArray = function(iterations)
{
	var counter = 0;
	if (0 < oolite.compareVersion("1.79")) { //slow and limited method, use before Oolite v1.79 only
		var ship;
		var ships;
		var dataKey;
		for(counter = 0;counter<iterations;counter++)
		{
			ships = system.addShips("player",64,[0,0,0], 1000000);
			if(ships) for( var j = 0; j < ships.length; j++ ) {
				ship = ships[j];
				if(ship) {
					dataKey = ship.dataKey;
					if (this.dataKeyArray.indexOf(dataKey) === -1)
						this.dataKeyArray.push(dataKey);
					if (ship.escorts && ship.escorts.length > 0){
						for (var k = 0; k < ship.escorts.length; k++)	{
							ship.escorts[k].remove(true);
						}
					}
					ship.remove(true);
				}
			}
		}
	} else { //Oolite v1.79 or later
		this.dataKeyArray = Ship.keysForRole( "player" );  //playable ships
	}
	this.dataKeyArrayBuilt = true;
	if (this.loggingEnabled)
	{
		var c = "";
		if( counter > 0 )  c = " in "+counter+" iterations.";
		log(this.name,"Unique player ship dataKeys found: "+ this.dataKeyArray.length + c );
		log(this.name,"dataKeys: "+this.dataKeyArray);
	}
}
		
this.storeCurrentShip = function(ship)
{
	if( !ship ) ship = player.ship;
	var counter;
	var storedShipArray = new Array;
	var stationAttributes;
	if (ship.dockedStation){stationAttributes = [ship.dockedStation.primaryRole, ship.dockedStation.dataKey, ship.dockedStation.position];} else {stationAttributes = [null,null,null];}
	storedShipArray.push([galaxyNumber,system.ID,clock.seconds,ship.displayName,system.name,stationAttributes,ship.price]); //storedShipArray[0]
	storedShipArray.push(ship.dataKey); // storedShipArray[1]
	if (!ship.aftWeapon){storedShipArray.push(ship.aftWeapon);} else {storedShipArray.push(ship.aftWeapon.equipmentKey);} // storedShipArray[2]
	if (!ship.forwardWeapon){storedShipArray.push(ship.forwardWeapon);} else {storedShipArray.push(ship.forwardWeapon.equipmentKey);} // storedShipArray[3]
	if (!ship.portWeapon){storedShipArray.push(ship.portWeapon);} else {storedShipArray.push(ship.portWeapon.equipmentKey);} // storedShipArray[4]
	if (!ship.starboardWeapon){storedShipArray.push(ship.starboardWeapon);} else {storedShipArray.push(ship.starboardWeapon.equipmentKey);} // storedShipArray[5]
	var missiles = ship.missiles;
	for(counter = 0;counter<missiles.length;counter++){missiles.splice(counter,1,missiles[counter].equipmentKey);}
	storedShipArray.push(missiles); // storedShipArray[6]
	var equipment = ship.equipment;
	var tempArray;
	for(counter = 0;counter<equipment.length;counter++)
	{
		tempArray = [equipment[counter].equipmentKey, ship.equipmentStatus(equipment[counter])];
		equipment.splice(counter,1,tempArray);
	}
	storedShipArray.push(equipment); // storedShipArray[7]
	var hold = new Array(17);
	for(counter = 0;counter<17;counter++){hold[counter] = manifest[this.cargoType[counter]];}
	storedShipArray.push(hold); // storedShipArray[8]
	storedShipArray.push(ship.fuel); //storedShipArray[9]
	var passengers = ship.passengers;
	for(counter = 0;counter<passengers.length;counter++)
	{
		tempArray = [passengers[counter].name,passengers[counter].start,passengers[counter].destination,(clock.seconds + passengers[counter].eta),passengers[counter].fee];
		passengers.splice(counter,1,tempArray);
	}
	storedShipArray.push(passengers); // storedShipArray[10]
	var subEnts = new Array;
	if (ship.subEntityCapacity > 0 && ship.subEntityCapacity !== ship.subEntities.length) // only store subEnts if at less than capacity.
	{for (counter = 0;counter < ship.subEntities.length; counter++){subEnts.push([ship.subEntities[counter].dataKey,ship.subEntities[counter].position.x,ship.subEntities[counter].position.y,ship.subEntities[counter].position.z]);}}
	storedShipArray.push(subEnts);//storedShipArray[11]
	
	if( ship == player.ship ) {
		storedShipArray.push(ship.serviceLevel);//storedShipArray[12]
		ship.script.$SSH_serviceLevel = ship.serviceLevel; //save for Carriers OXP
	} else if( ship.script ) storedShipArray.push(ship.script.$SSH_serviceLevel);
	
	storedShipArray.push(ship.entityPersonality);//storedShipArray[13]
	
	//EscortDeck specific data in storedShipArray[14]
	var u = 0;
	if( ship == player.ship ) u = 1; //player ship is always usable
	else if( ship.script ) u = ship.script.$EscortDeckUsable;
	var b = ship.boundingBox; //mass and sizes for TooLarge check and salvage price calculation
	storedShipArray.push({usable:u, mass:ship.mass, x:b.x, y:b.y, z:b.z});//storedShipArray[14]

	
    //the following section contain player-only data
    if( ship == player.ship ) {

	// this section is to record OXP equipment/weapon missionVariables or anything else needed for OXP compatibility that needs to be reinstated on ship restore.
	// cim's New Cargoes.
	var newCargoesManifest = null;
	if (worldScripts["CargoTypeExtension"]) {newCargoesManifest = worldScripts["CargoTypeExtension"].suspendPlayerManifest();}
	storedShipArray.push(newCargoesManifest); // storedShipArray[15]
	// thargoid's Hypercargo
	var hyperCargoMemory = null;
	var hyperCargoNewCargoes = null;
	var hyperCargoFailChance = null;
	if(ship.equipmentStatus("EQ_HYPERCARGO") === "EQUIPMENT_OK" && worldScripts["HyperCargo"])
	{ 
		if (missionVariables.hyperCargoMemory){hyperCargoMemory = missionVariables.hyperCargoMemory;}
		if (missionVariables.hyperCargoNewCargoes){hyperCargoNewCargoes = missionVariables.hyperCargoNewCargoes;}
		if (missionVariables.hyperCargoFailChance){hyperCargoFailChance = missionVariables.hyperCargoFailChance;}
	}
	storedShipArray.push([hyperCargoMemory,hyperCargoNewCargoes,hyperCargoFailChance]); // storedShipArray[16]
	// thargoids Vortex
	var name;
	var missileBays = new Array;
	var cargoBays = new Array;
	var nccargoBays = new Array;
	if (worldScripts["vortex_player.js"])
	{
		if (ship.name === "Vortex" || ship.name === "Maelstrom")
		{
			if (missionVariables.multiBay_storeShipName) {name = missionVariables.multiBay_storeShipName;}
			//if (missionVariables.multiBay_currentMissile) {missileBays.push(missionVariables.multiBay_currentMissile);}
			//if (missionVariables.multiBay_currentCargo) {cargoBays.push(missionVariables.multiBay_currentCargo);}
			for(counter = 0;counter<5;counter++)
			{
				if (missionVariables["multiBay_missileBay" + counter]){missileBays.push(missionVariables["multiBay_missileBay" + counter]);}
				if (missionVariables["multiBay_cargoBay" + counter]){cargoBays.push(missionVariables["multiBay_cargoBay" + counter]);}
				if (missionVariables["multiBay_ncBay"+counter]){nccargoBays.push(missionVariables["multiBay_ncBay"+counter]);}
			}
		}
	}
	storedShipArray.push([name,missileBays,cargoBays,nccargoBays]); // storedShipArray[17]
	// thargoid's APRIL
	var aprilMemory = null;
	var aprilExpanded = null;
	if (worldScripts["april_worldScript.js"] && ship.equipmentStatus("EQ_APRIL") !== "EQUIPMENT_UNAVAILABLE")
	{
		worldScripts["april_worldScript.js"].playerWillSaveGame("ship_storage_helper");
		if (missionVariables.aprilMemory){aprilMemory = missionVariables.aprilMemory;}
		if (missionVariables.aprilExpanded){aprilExpanded = missionVariables.aprilExpanded;}
	}
	storedShipArray.push([aprilMemory,aprilExpanded]); // storedShipArray[18]
	// thargoid's IronHide
	var ironHidePercentage = null;
	var ironHideMilFlag = null;
	if (worldScripts["IronHide Armour Script"] && ship.equipmentStatus("EQ_IRONHIDE") !== "EQUIPMENT_UNAVAILABLE")
	{
		if (missionVariables.ironHide_percentage){ironHidePercentage = missionVariables.ironHide_percentage;}
		if (missionVariables.ironHide_milFlag){ironHideMilFlag = missionVariables.ironHide_milFlag;}
	}
	storedShipArray.push([ironHidePercentage,ironHidePercentage]); // storedShipArray[19]
	// ramirez' Missiles & Bombs
	var rmbChaffCount = null;
	if (worldScripts["Missiles & Bombs"] && ship.equipmentStatus("EQ_RMB_CHAFF_LAUNCHER") !== "EQUIPMENT_UNAVAILABLE")
	{
		if (missionVariables.rmb_chaff_count){rmbChaffCount = missionVariables.rmb_chaff_count;}
	}
	storedShipArray.push(rmbChaffCount); //storedShipArray[20]
	// mcclane's Railgun
	var railgunType = null;
	var railgunProjectile = null;
	var railgunProjectiles = null;
	if (worldScripts["railgun"])
	{
		worldScripts["railgun"].playerWillSaveGame();
		if (missionVariables.railgun_type){railgunType = missionVariables.railgun_type;}
		if (missionVariables.railgun_projectile){railgunProjectile = missionVariables.railgun_projectile;}
		if (missionVariables.railgun_projectiles){railgunProjectiles = missionVariables.railgun_projectiles;}
	}
	storedShipArray.push([railgunType,railgunProjectile,railgunProjectiles]); // storedShipArray[21]
	// thargoid's targetAutoLock Plus
	var targetAutolock = null;
	if (worldScripts["targetAutolock"] && missionVariables.targetAutolock){targetAutolock = missionVariables.targetAutolock;}
	storedShipArray.push(targetAutolock); // storedShipArray[22]
	// eric's targetReticleSensitive
	var reticleTargetSensitive = null;
	if (worldScripts["reticle_target_sensitive"] && missionVariables.reticleTargetSensitive){reticleTargetSensitive = missionVariables.reticleTargetSensitive;}
	storedShipArray.push(reticleTargetSensitive); // storedShipArray[23]
	// thargoid's Armoury
	var CT_thargonCount = null;
	if (worldScripts["CT_Script"] && missionVariables.CT_thargonCount){CT_thargonCount = missionVariables.CT_thargonCount;}
	storedShipArray.push(CT_thargonCount); // storedShipArray[24]
	// thargoid's Aquatics
	var aquatics_guardianCount = null;
	if (worldScripts["aquatics_equipment"] && missionVariables.aquatics_guardianCount){aquatics_guardianCount = missionVariables.aquatics_guardianCount;}
	storedShipArray.push(aquatics_guardianCount); // storedShipArray[25]
	// smiv's Battle-Damage
	var BattleDamage_status = null;
	if ((worldScripts["Battle Damage"] || worldScripts["Battle Damage SS"]) && missionVariables.BattleDamage_status){BattleDamage_status = missionVariables.BattleDamage_status}
	storedShipArray.push(BattleDamage_status); // storedShipArray[26]
	// murphy's Respray for Griffs
	var favourite = null;
	var favouriteRGB = null;
	var currentRespray = null;
	var currentDecal = null;
	if (worldScripts["Respray_for_Griffs.js"])
	{
		if (worldScripts["Respray_for_Griffs.js"].currentRespray){currentRespray = worldScripts["Respray_for_Griffs.js"].currentRespray;}
		if (worldScripts["Respray_for_Griffs.js"].favourite){favourite = worldScripts["Respray_for_Griffs.js"].favourite;}
		if (worldScripts["Respray_for_Griffs.js"].favouriteRGB){favouriteRGB = worldScripts["Respray_for_Griffs.js"].favouriteRGB;}
		if (worldScripts["Respray_for_Griffs.js"].currentDecal){currentDecal = worldScripts["Respray_for_Griffs.js"].currentDecal;}
	}
	storedShipArray.push([favourite,currentRespray,favouriteRGB,currentDecal]); // storedShipArray[27]
	// finally log contents of array and return stringified copy.
	if (this.loggingEnabled){
		log(this.name,"storedShipArray:" + storedShipArray + "storedShipArray.length" + storedShipArray.length);
	}
	
    } //end of player-only section

    return JSON.stringify(storedShipArray);
}

this.restoreStoredShip = function(storedShipArray, ship)
{
	if( !ship ) ship = player.ship;
	if (!storedShipArray || typeof(storedShipArray) !== "string")
	{
		if (this.loggingEnabled){log(this.name,"Error - restoreStoredShip called with invalid parameter.");}
		return "Error-Invalid Parameter, expected String";
	}
	storedShipArray = JSON.parse(storedShipArray); // destringify storedShipArray
	if (this.loggingEnabled){log(this.name,"storedShipArray:" + storedShipArray + "storedShipArray.length" + storedShipArray.length);}
	var extraNulls = this.expectedLength - storedShipArray.length;
	if (extraNulls < 0) // storedShipArray is bigger than expected - should not happen
	{
		if (this.loggingEnabled){log(this.name,"Error - storedShipArray longer than expected length.");}
		if (this.loggingEnabled){log(this.name,"expectedLength" + this.expectedLength + "storedShipArray.length" + storedShipArray.length);}
		return "Error-Invalid Parameter, array incorrect length";
	}
	while (extraNulls > 0) // add extra nulls if smaller than expected (may have been created with older version of OXP)
	{
		storedShipArray.push(null);
		extraNulls--;
		if (this.loggingEnabled){log(this.name,"Adding nulls.....expectedLength: " + this.expectedLength + " storedShipArray.length: " + storedShipArray.length);}
	}
	// replace current ship with restored ship
	if (ship == player.ship && !player.replaceShip(storedShipArray[1],storedShipArray[13]))
	{
		if (this.loggingEnabled){log(this.name,"Cannot restore stored ship with dataKey " + storedShipArray[1] + ". dataKey not valid (suspect missing OXP)");}
		return "Error-Invalid dataKey";
	}
	// reinstate weapons
	ship.aftWeapon = storedShipArray[2];
	ship.forwardWeapon = storedShipArray[3];
	ship.portWeapon = storedShipArray[4];
	ship.starboardWeapon = storedShipArray[5];
	//reinstate pylons
	var counter;
	var counter1;
	var test;
	var missiles = ship.missiles;
	if (this.loggingEnabled){log(this.name,"Remove missiles automatically awarded on replaceShip: "+missiles);};
	for(counter = 0;counter<missiles.length;counter++){ship.removeEquipment(missiles[counter]);}
	missiles = storedShipArray[6];
	if (this.loggingEnabled){log(this.name,"Add missiles from storedShip: "+missiles);}
	for(counter = 0;counter<missiles.length;counter++){ship.awardEquipment(missiles[counter]);}
	//reinstate equipment
	if (worldScripts["Breakable_Energy_Unit"]){worldScripts["Breakable_Energy_Unit"].shipRestore = true;}
	if (worldScripts["Breakable_Engines"]){worldScripts["Breakable_Engines"].shipRestore = true;}
	if (worldScripts["Breakable_HUD_IFF_Scanner"]){worldScripts["Breakable_HUD_IFF_Scanner"].shipRestore = true;}
	if (worldScripts["Breakable_Shield_Generators"]){worldScripts["Breakable_Shield_Generators"].shipRestore = true;}
	if (worldScripts["Breakable_TorusDrive"]){worldScripts["Breakable_TorusDrive"].shipRestore = true;}
	if (worldScripts["Breakable_WitchDrive"]){worldScripts["Breakable_WitchDrive"].shipRestore = true;}
	var equipment = ship.equipment;
	if (this.loggingEnabled){log(this.name,"Equipment automatically awarded on replaceShip: "+equipment);}
	var tempArray;
	var savedEQ = storedShipArray[7];
	if (this.loggingEnabled){log(this.name,"Equipment from storedShip: "+savedEQ);}
	for(counter = 0;counter<equipment.length;counter++)
	{
		tempArray = [equipment[counter].equipmentKey, ship.equipmentStatus(equipment[counter])];
		for(counter1 = 0;counter1<savedEQ.length;counter1++)
		{
			if (savedEQ[counter1][0] === tempArray[0])
			{
				tempArray.push(true);
				tempArray.push(counter1);
				if (this.loggingEnabled){log(this.name,"Equipment: "+tempArray[0]+" from storedShip already fitted.");}
				if (savedEQ[counter1][1] === "EQUIPMENT_DAMAGED"){ship.setEquipmentStatus(tempArray[0],"EQUIPMENT_DAMAGED");if (this.loggingEnabled){log(this.name,"Setting status to: EQUIPMENT_DAMAGED");}}
				break;
			}
		}
		if (!tempArray[2])
		{
			if (this.loggingEnabled){log(this.name,"Not in storedShip equipment - removing equipment item: "+tempArray[0]);}
			ship.removeEquipment(tempArray[0]);
			continue;
		}
		else 
		{
			savedEQ.splice(tempArray[3],1);
			if (this.loggingEnabled){log(this.name,"Updated storedShip equipment list: "+savedEQ);}
		}
	}
	if (this.loggingEnabled){log(this.name,"Awarding remaining equipment from storedShip: "+savedEQ);}
	for(counter = 0;counter<savedEQ.length;counter++)
	{
		ship.awardEquipment(savedEQ[counter][0]);
		if(savedEQ[counter][1] === "EQUIPMENT_DAMAGED"){ship.setEquipmentStatus(savedEQ[counter][0],savedEQ[counter][1])};
	}
	if (worldScripts["Breakable_Energy_Unit"]){delete worldScripts["Breakable_Energy_Unit"].shipRestore;}
	if (worldScripts["Breakable_Engines"]){delete worldScripts["Breakable_Engines"].shipRestore;}
	if (worldScripts["Breakable_HUD_IFF_Scanner"]){delete worldScripts["Breakable_HUD_IFF_Scanner"].shipRestore;}
	if (worldScripts["Breakable_Shield_Generators"]){delete worldScripts["Breakable_Shield_Generators"].shipRestore;}
	if (worldScripts["Breakable_TorusDrive"]){delete worldScripts["Breakable_TorusDrive"].shipRestore;}
	if (worldScripts["Breakable_WitchDrive"]){delete worldScripts["Breakable_WitchDrive"].shipRestore;}
	if (ship.aftShield !== ship.maxAftShield){ship.aftShield = ship.maxAftShield;}
	if (ship.forwardShield !== ship.maxForwardShield){ship.forwardShield = ship.maxForwardShield;}
	// reinstate hold/manifest
	var hold = storedShipArray[8];
	for(counter = 0;counter<17;counter++){manifest[this.cargoType[counter]] = hold[counter];}
	// reinstate fuel
	ship.fuel = storedShipArray[9];
	// reinstate passengers
	var passengers = storedShipArray[10];
	for(counter = 0;counter<passengers.length;counter++)
	{
		if (clock.seconds >= passengers[counter][3]){if (this.loggingEnabled){log(this.name,"Cannot re-instate passenger record: "+passengers[counter]+" as passenger contract arrival time is in the past");};player.decreasePassengerReputation();continue;} 
		if (this.loggingEnabled){log(this.name,"Attempting to add passenger: "+passengers[counter]);}
		var test = ship.addPassenger(passengers[counter][0],passengers[counter][1],passengers[counter][2],passengers[counter][3],passengers[counter][4]);
		if (!test && this.loggingEnabled){log(this.name,"Failed to add passenger: "+passengers[counter]);}
	}
	// check for subents to be removed
	var subEnts = storedShipArray[11]; // will only exist if less subEnts than capacity when ship was stored.
	if (subEnts && subEnts.length>0)
	{
		var currentSubEnts = ship.subEntities;
		if (this.loggingEnabled){log(this.name,"Updating subEnts. current subEnts: "+ currentSubEnts +" storedShip subEnts: "+subEnts);}
		for (counter = 0;counter<currentSubEnts.length;counter++)
			{
				currentSubEnts[counter].toBeRemoved = true;
			    for(counter1 = 0;counter1<subEnts.length;counter1++)
				{
					if (subEnts[counter1][0] === currentSubEnts[counter].dataKey && subEnts[counter1][1] === currentSubEnts[counter].position.x && subEnts[counter1][2] === currentSubEnts[counter].position.y && subEnts[counter1][3] === currentSubEnts[counter].position.z){delete currentSubEnts[counter].toBeRemoved;}
				}
				if (currentSubEnts[counter].toBeRemoved){delete currentSubEnts[counter].toBeRemoved; currentSubEnts[counter].remove(true);}
			}
		if (this.loggingEnabled){log(this.name,"Updating subEnts complete. Current subEnts: "+ ship.subEntities +" storedShip subEnts: "+subEnts);}
	}

	// reinstate serviceLevel
	if( this.loggingEnabled ) log(this.name,"Updating ship.serviceLevel: "+ storedShipArray[12]);
	if( ship == player.ship ) ship.serviceLevel = storedShipArray[12];
	else if( ship.script ) s.script.$SSH_serviceLevel = storedShipArray[12];
	
	//storedShipArray[13] contain entityPersonality whixh is used up in replaceShip() above
	
	//EscortDeck specific data in storedShipArray[14]
	if( storedShipArray[14] ) {
		if( ship.script ) ship.script.$EscortDeckUsable = storedShipArray[14].usable;
		//ship mass and sizes are read-only and set by the core from the ship model
	}
	
	
    //the following section contain player-only data
    if( ship == player.ship ) {
		
	// reinstate OXP saved information.
	// cim's New Cargoes
	if (worldScripts["CargoTypeExtension"]) 
	{
		if (this.loggingEnabled){log(this.name,"Restoring New Cargoes special manifest: "+storedShipArray[15]);}
		worldScripts["CargoTypeExtension"].restorePlayerManifest(storedShipArray[15]);
	}
	// thargoid's HyperCargo
	if(ship.equipmentStatus("EQ_HYPERCARGO") === "EQUIPMENT_OK" && worldScripts["HyperCargo"])
	{ 
	  if (this.loggingEnabled){log(this.name,"Restoring Hypercargo missionVariables: "+storedShipArray[16]);}
	  missionVariables.hyperCargoMemory = storedShipArray[16][0];
	  missionVariables.hyperCargoNewCargoes = storedShipArray[16][1];
	  missionVariables.hyperCargoFailChance = storedShipArray[16][2];
	  worldScripts["HyperCargo"].startUp();
	}
	// thargoids Vortex
	if (worldScripts["vortex_player.js"])
	{
		if (ship.name === "Vortex" || ship.name === "Maelstrom")
		{
			if (this.loggingEnabled){log(this.name,"Restoring Vortex/Maelstrom missionVariables: "+storedShipArray[17]);}
			if (storedShipArray[17][0]){missionVariables.multiBay_storeShipName = storedShipArray[17][0];}
			//if (storedShipArray[17][1][0]){missionVariables.multiBay_currentMissile = storedShipArray[17][1][0];}
			//if (storedShipArray[17][2][0]){missionVariables.multiBay_currentCargo = storedShipArray[17][2][0];}
			for(counter = 0;counter<5;counter++)
			{
				if (storedShipArray[17][1][counter]){missionVariables["multiBay_missileBay"+counter] = storedShipArray[17][1][counter];}
				if (storedShipArray[17][2][counter]){missionVariables["multiBay_cargoBay"+counter] = storedShipArray[17][2][counter];}
				if (storedShipArray[17][3][counter]){missionVariables["multiBay_ncBay"+counter] = storedShipArray[17][3][counter];}
			}
		}
	}
	// thargoid's APRIL
	if (worldScripts["april_worldScript.js"] && ship.equipmentStatus("EQ_APRIL") !== "EQUIPMENT_UNAVAILABLE")
	{
		if (this.loggingEnabled){log(this.name,"Restoring APRIL missionVariables: "+storedShipArray[18]);}
		missionVariables.aprilMemory = storedShipArray[18][0];
		missionVariables.aprilExpanded = storedShipArray[18][1];
		worldScripts["april_worldScript.js"].startUp();
	}
	// thargoid's IronHide
	if (worldScripts["IronHide Armour Script"] && ship.equipmentStatus("EQ_IRONHIDE") !== "EQUIPMENT_UNAVAILABLE")
	{
		if (this.loggingEnabled){log(this.name,"Restoring IronHide missionVariables: "+storedShipArray[19]);}
		missionVariables.ironHide_percentage = storedShipArray[19][0];
		missionVariables.ironHide_milFlag = storedShipArray[19][1];
		worldScripts["IronHide Armour Script"].startUp();
	}
	// ramirez's Missiles & Bombs
	if (worldScripts["Missiles & Bombs"] && ship.equipmentStatus("EQ_RMB_CHAFF_LAUNCHER") !== "EQUIPMENT_UNAVAILABLE"){if (this.loggingEnabled){log(this.name,"Restoring Missile & Bombs missionVariable: "+storedShipArray[20]);} missionVariables.rmb_chaff_count = storedShipArray[20];}
	// mcclane's railgun
	if (worldScripts["railgun"])
	{
		if (this.loggingEnabled){log(this.name,"Restoring railgun missionVariables: "+storedShipArray[21]);}
		missionVariables.railgun_type = storedShipArray[21][0];
		missionVariables.railgun_projectile = storedShipArray[21][1];
		missionVariables.railgun_projectiles = storedShipArray[21][2];
		worldScripts["railgun"].startUp();
	}
	// thargoid's targetAutoLock Plus.
	if (worldScripts["targetAutolock"]){if (this.loggingEnabled){log(this.name,"Restoring targetAutoLock Plus missionVariables: "+storedShipArray[22]);} missionVariables.targetAutolock = storedShipArray[22];}
	// eric's targetReticleSensitive
	if (worldScripts["reticle_target_sensitive"])
	{
		if (this.loggingEnabled){log(this.name,"Restoring targetReticle missionVariables: "+storedShipArray[23]);}
		missionVariables.reticleTargetSensitive = storedShipArray[23];
		worldScripts["reticle_target_sensitive"].startUp();
	}
	// thargoid's Armoury
	if (worldScripts["CT_Script"])
	{
		if (this.loggingEnabled){log(this.name,"Restoring Armoury missionVariable: "+storedShipArray[24]);}
		missionVariables.CT_thargonCount = storedShipArray[24];
		worldScripts["CT_Script"].startUp();
	}
	// thargoid's Aquatics
	if (worldScripts["aquatics_equipment"]){if (this.loggingEnabled){log(this.name,"Restoring Aquatics missionVariable: "+storedShipArray[25]);} missionVariables.aquatics_guardianCount = storedShipArray[25];}
	// smiv's Battle Damage
	if (worldScripts["Battle Damage"] || worldScripts["Battle Damage SS"]){if (this.loggingEnabled){log(this.name,"Restoring Battle Damage missionVariable: "+storedShipArray[26]);} 
	missionVariables.BattleDamage_status = storedShipArray[26];}
	// Respray for Griff's
	if (worldScripts["Respray_for_Griffs.js"])
	{
		if (this.loggingEnabled){log(this.name,"Restoring Respray for Griff's missionVariable: "+storedShipArray[27]);}
		worldScripts["Respray_for_Griffs.js"].favourite = storedShipArray[27][0];
		worldScripts["Respray_for_Griffs.js"].currentRespray = storedShipArray[27][1];
		worldScripts["Respray_for_Griffs.js"].favouriteRGB = storedShipArray[27][2];
		worldScripts["Respray_for_Griffs.js"].currentDecal = storedShipArray[27][3];
		worldScripts["Respray_for_Griffs.js"].playerWillSaveGame();
		worldScripts["Respray_for_Griffs.js"].startUp();
	}

    } //end of player-only section

    return true;
}

this.disableVortexPBNSFunc = function(context, ship)
{
	if (!worldScripts["vortex_player.js"] || this.storeVortexFunc) {return false;}
	if( !ship ) ship = player.ship;
	
	this.storeVortexFunc = new Object;
	this.storeVortexFunc = worldScripts["vortex_player.js"].playerBoughtNewShip;
	if (context === "delete") {delete worldScripts["vortex_player.js"].playerBoughtNewShip; if (this.loggingEnabled){log(this.name,"Deleted Vortex this.playerBoughtNewShip");}return true;}
	if (context === "retainName")
	{
		this.replaceVortexFunc = new Object;
		this.replaceVortexFunc = function (ship) {
		this.stopMissileTimer();
		this.resetVortex();
		if (ship.name === "Vortex"){
        this.storeMissileBay(missionVariables.multiBay_currentMissile);
        this.startUp();
		this.shipTitle = missionVariables.multiBay_storeShipName;
        mission.runScreen({title: "Ship Specifications (Vortex)", model: "vortex_player"});
        this.AIName = "MAC" + Math.floor(Math.random() * 0.999999 * 10) + Math.floor(Math.random() * 0.999999 * 10) + Math.floor(Math.random() * 0.999999 * 10) + Math.floor(Math.random() * 0.999999 * 10);
        mission.addMessageText("Designation: Vortex - " + this.shipTitle);
        mission.addMessageText("MAC registration: " + this.AIName + "\n");
        mission.addMessageText("Shipyard: Aquarian Shipbuilding Corporation                                      Special Capabilities:");
        mission.addMessageText("Source: Ex-Military (blockade runner)                                                     Military grade AI");
        mission.addMessageText("Max speed: 450                                                                                     Damage auto-repair");
        mission.addMessageText("Cargo capacity: 5x 30t                                                                    Missile defense screen");
        mission.addMessageText("Missile capacity: 5x 5 pylons                                                  Regenerative fuel injection");
        mission.addMessageText("Energy: 6 banks                                                                             Multi-bay cargo storage");
        mission.addMessageText("Pitch/Roll/Yaw: 1.3/1.5/1.5                                                            Switchable missile bays");
        mission.addMessageText("\n\n\nGreetings Commander " + expandDescription("[commander_name]") + ".\n\nI am " + this.AIName + ", the military AI construct of your new ASC Vortex craft, the " + this.shipTitle + ". Self-diagnostics shows all systems functional and ready - I await your command.");
        mission.addMessageText("\nEnd of Line.");}}
		worldScripts["vortex_player.js"].playerBoughtNewShip = this.replaceVortexFunc;
		if (this.loggingEnabled){log(this.name,"Modified Vortex this.playerBoughtNewShip");}
		return true;
	}
	return false;
}

this.enableVortexPBNSFunc = function()
{
	if (!worldScripts["vortex_player.js"] || !this.storeVortexFunc) {return false;}
	worldScripts["vortex_player.js"].playerBoughtNewShip = this.storeVortexFunc;
	delete this.storeVortexFunc;
	if (this.loggingEnabled){log(this.name,"Reinstated original Vortex this.playerBoughtNewShip");}
	return true;
}

this.disableMaelstromPBNSFunc = function(context, ship)
{
	if (!worldScripts["vortex_maelstrom.js"] || this.storeMaelstromFunc) {return false;}
	if( !ship ) ship = player.ship;

	this.storeMaelstromFunc = new Object;
	this.storeMaelstromFunc = worldScripts["vortex_maelstrom.js"].playerBoughtNewShip;
	if (context === "delete") {delete worldScripts["vortex_maelstrom.js"].playerBoughtNewShip;if (this.loggingEnabled){log(this.name,"Deleted Maelstrom this.playerBoughtNewShip");} return true;}
	if (context === "retainName")
	{
		this.replaceMaelstromFunc = new Object;
		this.replaceMaelstromFunc = function (ship) {
		this.stopMissileTimer();
		this.resetShip();
		if (ship.name === "Maelstrom") {
		this.storeMissileBay(missionVariables.multiBay_currentMissile);
        this.startUp();
		this.shipTitle = missionVariables.multiBay_storeShipName;
        mission.runScreen({title: "Ship Specifications (Maelstrom)", model: "vortex_maelstrom_player"});
        this.AIName = "MAC" + Math.floor(Math.random() * 0.999999 * 10) + Math.floor(Math.random() * 0.999999 * 10) + Math.floor(Math.random() * 0.999999 * 10) + Math.floor(Math.random() * 0.999999 * 10);
        mission.addMessageText("Designation: Maelstrom - " + this.shipTitle);
        mission.addMessageText("MAC registration: " + this.AIName + "\n");
        mission.addMessageText("Shipyard: Aquarian Shipbuilding Corporation                                Special Capabilities:");
        mission.addMessageText("Source: Ex-Military (Armoured Transport)                                              Military grade AI");
        mission.addMessageText("Max speed: 250                                                                                     Damage auto-repair");
        mission.addMessageText("Cargo capacity: 3x 100t                                                                  Missile defense screen");
        mission.addMessageText("Missile capacity: 3x 4 pylons                                                  Regenerative fuel injection");
        mission.addMessageText("Energy: 10 banks                                                                            Multi-bay cargo storage");
        mission.addMessageText("Pitch/Roll/Yaw: 1.0/1.2/1.2                                                            Switchable missile bays");
        mission.addMessageText("\n\n\nGreetings Commander " + expandDescription("[commander_name]") + ".\n\nI am " + this.AIName + ", the military AI construct of your new ASC Maelstrom craft, the " + this.shipTitle + ". Self-diagnostics shows all systems functional and ready - I await your command.");
        mission.addMessageText("\nEnd of Line.");}}
	worldScripts["vortex_maelstrom.js"].playerBoughtNewShip = this.replaceMaelstromFunc;
	if (this.loggingEnabled){log(this.name,"Modified Maelstrom this.playerBoughtNewShip");}
	return true;
	}
	return false;
}

this.enableMaelstromPBNSFunc = function()
{
	if (!worldScripts["vortex_maelstrom.js"] || !this.storeMaelstromFunc) {return false;}
	worldScripts["vortex_maelstrom.js"].playerBoughtNewShip = this.storeMaelstromFunc;
	delete this.storeMaelstromFunc;
	if (this.loggingEnabled){log(this.name,"Reinstated original Maelstrom this.playerBoughtNewShip");}
	return true;
}

this.disableTCATPBNSFunc = function()
{
	if (!worldScripts["TCAT_missionScript"] || this.storeTCATFunc) {return false;}
	this.storeTCATFunc = new Object;
	this.storeTCATFunc = worldScripts["TCAT_missionScript"].playerBoughtNewShip;
	delete worldScripts["TCAT_missionScript"].playerBoughtNewShip;
	if (this.loggingEnabled){log(this.name,"Deleted TCAT this.playerBoughtNewShip");}
	return true;
}

this.enableTCATPBNSFunc = function()
{
	if (!worldScripts["TCAT_missionScript"] || !this.storeTCATFunc) {return false;}
	worldScripts["TCAT_missionScript"].playerBoughtNewShip = this.storeTCATFunc;
	delete this.storeTCATFunc;
	if (this.loggingEnabled){log(this.name,"Reinstated original TCAT this.playerBoughtNewShip");}
	return true;
}
	